import path from "path";
import fs from "fs";
import { getIconCSS, getIconData } from "@iconify/utils";
import { createRequire } from "module";
import { availableCollectionNames } from "./collections.js";
import { parseIconSet } from "@iconify/utils";
import { defu } from "defu";
const req = createRequire(import.meta.url);
export const localResolve = (cwd, id) => {
  try {
    const resolved = req.resolve(id, { paths: [cwd] });
    return resolved;
  } catch {
    return null;
  }
};
function callerPath() {
  const error = new Error();
  const stack = error.stack?.split("\n");
  const data = stack.find(
    (line) => !line.trim().startsWith("Error") && !line.includes("(") && !line.includes(")")
  );
  if (!data) {
    return null;
  }
  const filePathPattern = new RegExp(
    // eslint-disable-next-line no-useless-escape
    /\s*at (\/.*|[a-zA-Z]:\\(?:([^<>:"\/\\|?*]*[^<>:"\/\\|?*.]\\|..\\)*([^<>:"\/\\|?*]*[^<>:"\/\\|?*.]\\?|..\\))?):\d+:\d+/i
  );
  const result = filePathPattern.exec(data);
  if (!result) {
    return null;
  }
  return result[1];
}
export const isPackageExists = (id) => {
  const p = callerPath();
  const cwd = p ? path.dirname(p) : process.cwd();
  return Boolean(localResolve(cwd, id));
};
export const getIconCollections = (include = "all") => {
  const p = callerPath();
  const cwd = p ? path.dirname(p) : process.cwd();
  const pkgPath = localResolve(cwd, "@iconify/json/package.json");
  if (!pkgPath) {
    if (Array.isArray(include)) {
      return include.reduce((result, name) => {
        const jsonPath = localResolve(cwd, `@iconify-json/${name}/icons.json`);
        if (!jsonPath) {
          throw new Error(
            `Icon collection "${name}" not found. Please install @iconify-json/${name} or @iconify/json`
          );
        }
        return {
          ...result,
          [name]: req(jsonPath)
        };
      }, {});
    }
    if (include === "all") {
      throw new Error(
        `All icon collections requested. Please install @iconify/json`
      );
    }
    return {};
  }
  const pkgDir = path.dirname(pkgPath);
  const files = fs.readdirSync(path.join(pkgDir, "json"));
  const collections = {};
  for (const file of files) {
    if (include === "all" || include.includes(file.replace(".json", ""))) {
      const json = req(path.join(pkgDir, "json", file));
      collections[json.prefix] = json;
    }
  }
  return collections;
};
export const getAutoIconCollections = (collections) => {
  if (Array.isArray(collections)) {
    return getIconCollections(collections);
  }
  if (typeof collections === "object")
    return collections;
  const names = collections === "all" ? "all" : availableCollectionNames.filter(
    (name) => isPackageExists(`@iconify-json/${name}`)
  );
  return getIconCollections(names);
};
export const getCustomCollections = (customCollections) => {
  if (typeof customCollections === "string") {
    const json = req(customCollections);
    return {
      [json.prefix]: json
    };
  }
  if (Array.isArray(customCollections)) {
    return customCollections.reduce((result, collection) => {
      if (typeof collection === "string") {
        const json = req(collection);
        return {
          ...result,
          [json.prefix]: json
        };
      }
      return {
        ...result,
        ...collection
      };
    }, {});
  }
  return customCollections;
};
export const generateIconComponent = (data, options) => {
  const css = getIconCSS(data, {});
  const rules = {};
  css.replace(/^\s+([^:]+):\s*([^;]+);/gm, (_, prop, value) => {
    if (prop === "width" || prop === "height") {
      rules[prop] = options.size === false ? "" : String(Number(options.size)) === options.size ? `${options.size}px` : `${options.size}`;
      options.size === false && delete rules[prop];
    } else {
      rules[prop] = value;
    }
    return "";
  });
  Object.assign(rules, { verticalAlign: "middle" });
  if (options.extraProperties) {
    Object.assign(rules, options.extraProperties);
  }
  return rules;
};
export const generateComponent = ({
  name,
  icons
}, options) => {
  const data = getIconData(icons, name);
  if (!data)
    return null;
  return generateIconComponent(data, options);
};
export const getAllIconComponents = (iconsTailwindPluginOptions) => {
  const {
    collections: propsCollections,
    customCollections = {},
    size = "1em",
    prefix = "i",
    extraProperties = {}
  } = iconsTailwindPluginOptions ?? {};
  const collections = defu(
    {},
    getAutoIconCollections(propsCollections),
    getCustomCollections(customCollections)
  );
  const components = {};
  const collectionPrefixes = [];
  if (prefix)
    components[prefix] = {};
  for (const colPrefix of Object.keys(collections)) {
    collectionPrefixes.push(prefix ? `${prefix}-${colPrefix}` : `${colPrefix}`);
    const collection = {
      ...collections[colPrefix],
      prefix: colPrefix
    };
    if (!prefix)
      components[colPrefix] = {};
    parseIconSet(collection, (name, data) => {
      if (!data || !name)
        return;
      const key = prefix ? `${colPrefix}-${name}` : `${name}`;
      components[prefix || colPrefix][key] = generateIconComponent(data, {
        size,
        extraProperties
      });
    });
  }
  return components;
};
export const getAllPrefixes = (iconsPluginOptions) => {
  const {
    collections: propsCollections,
    customCollections = {},
    prefix = "i"
  } = iconsPluginOptions ?? {};
  const collections = defu(
    {},
    getAutoIconCollections(propsCollections),
    getCustomCollections(customCollections)
  );
  const collectionPrefixes = [];
  for (const colPrefix of Object.keys(collections)) {
    collectionPrefixes.push(prefix ? `${prefix}-${colPrefix}` : `${colPrefix}`);
  }
  return collectionPrefixes;
};
