/**
 * Icon dimensions.
 *
 * Used in:
 *  icon (as is)
 *  alias (overwrite icon's properties)
 *  root of JSON file (default values)
 */
export interface IconifyDimenisons {
    left?: number;
    top?: number;
    width?: number;
    height?: number;
}
/**
 * Icon transformations.
 *
 * Used in:
 *  icon (as is)
 *  alias (merged with icon's properties)
 */
export interface IconifyTransformations {
    rotate?: number;
    hFlip?: boolean;
    vFlip?: boolean;
}
/**
 * Combination of dimensions and transformations.
 */
export interface IconifyOptional extends IconifyDimenisons, IconifyTransformations {
}
/**
 * Alias.
 */
export interface IconifyAlias extends IconifyOptional {
    parent: string;
}
/**
 * Icon.
 */
export interface IconifyIcon extends IconifyOptional {
    body: string;
}
/**
 * Icon with optional parameters that are provided by API and affect only search
 */
interface APIIconAttributes {
    hidden?: boolean;
}
export interface ExtendedIconifyIcon extends IconifyIcon, APIIconAttributes {
}
export interface ExtendedIconifyAlias extends IconifyAlias, APIIconAttributes {
}
/**
 * "icons" field of JSON file.
 */
export interface IconifyIcons {
    [index: string]: ExtendedIconifyIcon;
}
/**
 * "aliases" field of JSON file.
 */
export interface IconifyAliases {
    [index: string]: ExtendedIconifyAlias;
}
/**
 * Icon set information block.
 */
export interface IconifyInfo {
    name: string;
    total?: number;
    version?: string;
    author: {
        name: string;
        url?: string;
    };
    license: {
        title: string;
        spdx?: string;
        url?: string;
    };
    samples?: string[];
    height?: number | number[];
    displayHeight?: number;
    category?: string;
    tags?: string[];
    palette?: boolean;
    hidden?: boolean;
}
/**
 * Optional themes, old format.
 *
 * Deprecated because format is unnecessary complicated. Key is meaningless, suffixes and prefixes are mixed together.
 */
export interface LegacyIconifyThemes {
    [index: string]: {
        title: string;
        prefix?: string;
        suffix?: string;
    };
}
/**
 * Characters used in font.
 */
export interface IconifyChars {
    [index: string]: string;
}
/**
 * Icon categories
 */
export interface IconifyCategories {
    [index: string]: string[];
}
/**
 * Meta data stored in JSON file, used for browsing icon set.
 */
export interface IconifyMetaData {
    info?: IconifyInfo;
    chars?: IconifyChars;
    categories?: IconifyCategories;
    themes?: LegacyIconifyThemes;
    prefixes?: Record<string, string>;
    suffixes?: Record<string, string>;
}
/**
 * JSON structure, contains only icon data
 */
export interface IconifyJSONIconsData extends IconifyDimenisons {
    prefix: string;
    provider?: string;
    icons: IconifyIcons;
    aliases?: IconifyAliases;
}
/**
 * JSON structure.
 *
 * All optional values can exist in root of JSON file, used as defaults.
 */
export interface IconifyJSON extends IconifyJSONIconsData, IconifyMetaData {
    lastModified?: number;
    not_found?: string[];
}
/**
 * Structure of exports '@iconify-json/*' packages.
 *
 * These are small packages, one per icon set, that split JSON structure into multiple files to reduce
 * amount of data imported from package.
 */
export interface IconifyJSONPackageExports {
    info: IconifyInfo;
    icons: IconifyJSON;
    metadata: IconifyMetaData;
    chars: IconifyChars;
}
export {};
